@@ -109,9 +109,9 @@ namespace Uml
             Dependency,
             Association,
             Association_Self,
-            Coll_Message_Asynchronous,
+            Coll_Mesg_Async,
             Seq_Message,
-            Coll_Message_Self,
+            Coll_Mesg_Self,
             Seq_Message_Self,
             Containment,
             Composition,
@@ -124,7 +124,7 @@ namespace Uml
             Category2Parent,
             Child2Category,
             Relationship,
-            Coll_Message_Synchronous,
+            Coll_Mesg_Sync,
             // enter new entries before this line
             Reserved,
             Unknown  =  - 1
@@ -135,7 +135,7 @@ namespace Uml
         Enum fromInt(int item);
         bool hasUMLRepresentation(Enum item);
     }
-    
+
     /**
      * Layout types.
      */
@@ -153,7 +153,19 @@ namespace Uml
         Enum fromString(const QString& item);
         Enum fromInt(int item);
     }
-    
+
+    /**
+     * Diagram property controlling display of stereotypes.
+     */
+    namespace ShowStereoType
+    {
+        enum Enum {
+            None = 0,    ///< no display of stereotype
+            Name,        ///< stereotype name only
+            Tags         ///< stereotype name and tagged values
+        };
+    }
+
     /**
      * Signature types.
      */
@@ -222,7 +234,8 @@ namespace Uml
             Asynchronous,
             Creation,
             Lost,
-            Found
+            Found,
+            Destroy,
         };
         QString toString(Enum item);
         Enum fromString(const QString& item);
@@ -262,6 +275,28 @@ namespace Uml
         Enum fromInt(int item);
     }
 
+    /**
+     * UML primitive types
+     */
+    namespace PrimitiveTypes
+    {
+        enum Enum {
+            String,
+            Boolean,
+            UnlimitedNatural,
+            Integer,
+            Real,
+            Reserved
+        };
+
+        const int n_types = int(Reserved);
+
+        QString toString(Enum item);
+        QString toString(int item);
+        Enum fromString(const QString& item, bool strict = false);
+        Enum fromInt(int item);
+    }
+
     /**
      * Supported programming languages.
      */
@@ -288,7 +323,7 @@ namespace Uml
             Tcl,
             Vala,
             XMLSchema,
-            Reserved
+            Reserved   // for UML Primitive Types (no code generator)
         };
         QString toString(Enum item);
         Enum fromString(const QString& item);
