@@ -5,7 +5,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   copyright (C) 2007 Jari-Matti Mäkelä <jmjm@iki.fi>                    *
- *   copyright (C) 2008-2014                                               *
+ *   copyright (C) 2008-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -35,12 +35,12 @@ public:
     /**
      * Save the XMI representation of this object
      */
-    virtual void saveToXMI (QDomDocument & doc, QDomElement & root);
+    virtual void saveToXMI1(QXmlStreamWriter& writer);
 
     /**
      * Load params from the appropriate XMI element node.
      */
-    virtual void loadFromXMI (QDomElement & root);
+    virtual void loadFromXMI1 (QDomElement & root);
 
     /**
      * Update the start and end text for this hierarchical codeblock.
