@@ -5,7 +5,7 @@
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   copyright (C) 2007 Jari-Matti Mäkelä <jmjm@iki.fi>                    *
- *   copyright (C) 2008-2014                                               *
+ *   copyright (C) 2008-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -28,6 +28,9 @@
 #include "dcodeclassfield.h"
 #include "dcodedocumentation.h"
 
+// qt/kde includes
+#include <QXmlStreamWriter>
+
 DCodeAccessorMethod::DCodeAccessorMethod (CodeClassField * field, CodeAccessorMethod::AccessorType type)
         : CodeAccessorMethod (field)
 {
@@ -42,10 +45,10 @@ DCodeAccessorMethod::~DCodeAccessorMethod ()
 {
 }
 
-void DCodeAccessorMethod::setAttributesOnNode (QDomDocument & doc, QDomElement & blockElement)
+void DCodeAccessorMethod::setAttributesOnNode (QXmlStreamWriter& writer)
 {
     // set super-class attributes
-    CodeAccessorMethod::setAttributesOnNode(doc, blockElement);
+    CodeAccessorMethod::setAttributesOnNode(writer);
 
     // set local attributes now
 }
@@ -121,7 +124,7 @@ void DCodeAccessorMethod::updateMethodDeclaration()
     DCodeClassField * dfield = dynamic_cast<DCodeClassField*>(getParentClassField());
 
     // Check for dynamic casting failure!
-    if (dfield == NULL)
+    if (dfield == 0)
     {
         uError() << "dfield: invalid dynamic cast";
         return;
