@@ -194,6 +194,13 @@ void PythonWriter::writeClass(UMLClassifier *c)
 
     //find an appropriate name for our file
     QString fileName = findFileName(c, QLatin1String(".py"));
+
+    // Do not generate files for classes that has a container
+    if (hasContainer(fileName)) {
+        emit codeGenerated(c, CodeGenerator::Skipped);
+        return;
+    }
+
     if (fileName.isEmpty()) {
         emit codeGenerated(c, false);
         return;
@@ -205,6 +212,7 @@ void PythonWriter::writeClass(UMLClassifier *c)
         return;
     }
     QTextStream h(&fileh);
+    h.setCodec("UTF-8");
 
     //////////////////////////////
     //Start generating the code!!
@@ -220,7 +228,6 @@ void PythonWriter::writeClass(UMLClassifier *c)
         h<<str<<m_endl;
     }
 
-    h << "# coding=" << h.codec()->name() << m_endl;
     // generate import statement for superclasses and take packages into account
     str = cleanName(c->name());
     QString pkg = cleanName(c->package());
@@ -266,7 +273,10 @@ void PythonWriter::writeClass(UMLClassifier *c)
 
     if (forceDoc() || !c->doc().isEmpty()) {
         h << m_indentation << "\"\"\"" << m_endl;
-        h << formatDoc(c->doc(), m_indentation + QLatin1Char(' ')) << m_endl;
+        if (!c->doc().isEmpty()) {
+            h << formatDoc(c->doc(), m_indentation + QLatin1Char(' ')) << m_endl;
+            h << m_endl;
+        }
         h << m_indentation << ":version:" << m_endl;
         h << m_indentation << ":author:" << m_endl;
         h << m_indentation << "\"\"\"" << m_endl << m_endl;
@@ -305,7 +315,10 @@ void PythonWriter::writeAttributes(UMLAttributeList atList, QTextStream &py)
         return;
     py << m_indentation << "\"\"\" ATTRIBUTES" << m_endl << m_endl;
     foreach (UMLAttribute *at, atList) {
-        py << formatDoc(at->doc(), m_indentation + QLatin1Char(' ')) << m_endl;
+        if (!at->doc().isEmpty()) {
+            py << formatDoc(at->doc(), m_indentation + QLatin1Char(' ')) << m_endl;
+            py << m_endl;
+        }
         Uml::Visibility::Enum vis = at->visibility();
         py << m_indentation << cleanName(at->name()) << "  ("
             << Uml::Visibility::toString(vis) << ")" << m_endl << m_endl ;
@@ -433,6 +446,16 @@ void PythonWriter::writeOperations(const QString& classname, UMLOperationList &o
     }//end for
 }
 
+/**
+ * Check if type is a container
+ * @param string      type that will be used
+ * @return            true if is a container
+ */
+bool PythonWriter::hasContainer(const QString &string)
+{
+    return string.contains(QStringLiteral("<")) && string.contains(QStringLiteral(">"));
+}
+
 /**
  * Return the programming language identifier.
  * @return   programming language id
