@@ -6,7 +6,7 @@
  *                                                                         *
  *   copyright (C) 2005                                                    *
  *   Richard Dale  <Richard_Dale@tipitina.demon.co.uk>                     *
- *   copyright (C) 2006-2014                                               *
+ *   copyright (C) 2006-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -29,19 +29,19 @@ RubyClassDeclarationBlock::~RubyClassDeclarationBlock ()
 /**
  * Save the XMI representation of this object
  */
-void RubyClassDeclarationBlock::saveToXMI (QDomDocument & doc, QDomElement & root)
+void RubyClassDeclarationBlock::saveToXMI1(QXmlStreamWriter& writer)
 {
-    QDomElement blockElement = doc.createElement(QLatin1String("rubyclassdeclarationblock"));
+    writer.writeStartElement(QLatin1String("rubyclassdeclarationblock"));
 
-    setAttributesOnNode(doc, blockElement);
+    setAttributesOnNode(writer);
 
-    root.appendChild(blockElement);
+    writer.writeEndElement();
 }
 
 /**
  * load params from the appropriate XMI element node.
  */
-void RubyClassDeclarationBlock::loadFromXMI (QDomElement & root)
+void RubyClassDeclarationBlock::loadFromXMI1 (QDomElement & root)
 {
     setAttributesFromNode(root);
 }
@@ -52,6 +52,7 @@ void RubyClassDeclarationBlock::loadFromXMI (QDomElement & root)
 void RubyClassDeclarationBlock::updateContent ()
 {
     RubyClassifierCodeDocument *parentDoc = dynamic_cast<RubyClassifierCodeDocument*>(getParentDocument());
+    Q_ASSERT(parentDoc);
     UMLClassifier *c = parentDoc->getParentClassifier();
     CodeGenerationPolicy * p = UMLApp::app()->commonPolicy();
     QString endLine = p->getNewLineEndingChars();
@@ -120,4 +121,3 @@ void RubyClassDeclarationBlock::init (RubyClassifierCodeDocument *parentDoc, con
     setEndText(QLatin1String("end"));
 }
 
-#include "rubyclassdeclarationblock.moc"
