@@ -6,7 +6,7 @@
  *                                                                         *
  *   copyright (C) 2005                                                    *
  *   Richard Dale  <Richard_Dale@tipitina.demon.co.uk>                     *
- *   copyright (C) 2006-2014                                               *
+ *   copyright (C) 2006-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -30,6 +30,7 @@
 
 // qt includes
 #include <QRegExp>
+#include <QXmlStreamWriter>
 
 /**
  * Constructor.
@@ -55,10 +56,10 @@ RubyCodeAccessorMethod::~RubyCodeAccessorMethod()
  * Set attributes of the node that represents this class
  * in the XMI document.
  */
-void RubyCodeAccessorMethod::setAttributesOnNode(QDomDocument& doc, QDomElement& blockElement)
+void RubyCodeAccessorMethod::setAttributesOnNode(QXmlStreamWriter& writer)
 {
     // set super-class attributes
-    CodeAccessorMethod::setAttributesOnNode(doc, blockElement);
+    CodeAccessorMethod::setAttributesOnNode(writer);
 
     // set local attributes now
 }
@@ -81,7 +82,7 @@ void RubyCodeAccessorMethod::updateContent()
     RubyCodeClassField * rubyfield = dynamic_cast<RubyCodeClassField*>(parentField);
 
     // Check for dynamic casting failure!
-    if (rubyfield == NULL)
+    if (rubyfield == 0)
     {
         uError() << "rubyfield: invalid dynamic cast";
         return;
@@ -146,7 +147,7 @@ void RubyCodeAccessorMethod::updateMethodDeclaration()
     RubyCodeClassField * rubyfield = dynamic_cast<RubyCodeClassField*>(getParentClassField());
 
     // Check for dynamic casting failure!
-    if (rubyfield == NULL)
+    if (rubyfield == 0)
     {
         uError() << "rubyfield: invalid dynamic cast";
         return;
