@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2014                                               *
+ *   copyright (C) 2002-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -19,13 +19,13 @@
  * 
  * @author Ralf Habacker <ralf.habacker@freenet.de>
  *
- * Bugs and comments to umbrello-devel@kde.org or http://bugs.kde.org
+ * Bugs and comments to umbrello-devel@kde.org or https://bugs.kde.org
  */
 class AutoLayoutOptionPage : public DialogPageBase, private Ui::AutoLayoutOptionPage
 {
     Q_OBJECT
 public:
-    explicit AutoLayoutOptionPage(QWidget* parent);
+    explicit AutoLayoutOptionPage(QWidget* parent = 0);
     virtual ~AutoLayoutOptionPage();
 
     void setDefaults();
