@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2016-2020                                               *
+ *   copyright (C) 2016-2021                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -13,7 +13,7 @@
 
 #include <QWidget>
 #include "singlepagedialogbase.h"
-#include "attribute.h"
+#include "instanceattribute.h"
 
 namespace Ui {
 class UMLInstanceAttributeDialog;
@@ -24,18 +24,15 @@ class UMLInstanceAttributeDialog : public SinglePageDialogBase
     Q_OBJECT
 
 public:
-    explicit UMLInstanceAttributeDialog(QWidget *parent, UMLAttribute *pAttribute);
+    explicit UMLInstanceAttributeDialog(QWidget *parent, UMLInstanceAttribute *pInstanceAttr);
     ~UMLInstanceAttributeDialog();
 
 private:
     Ui::UMLInstanceAttributeDialog *ui;
-    UMLAttribute *m_pAttribute;
+    UMLInstanceAttribute *m_pInstanceAttr;
 
 protected:
     bool apply();
-
-public slots:
-    void slotNameChanged(const QString &text);
 };
 
 #endif // UMLINSTANCEATTRIBUTEDIALOG_H
