@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2020                                               *
+ *   copyright (C) 2002-2021                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -563,6 +563,13 @@ QString UMLObject::stereotype(bool includeAdornments /* = false */) const
     return m_pStereotype->name(includeAdornments);
 }
 
+/**
+ * Returns the concrete values of stereotype attributes.
+ */
+QStringList & UMLObject::tags() {
+    return m_TaggedValues;
+}
+
 /**
  * Return the package(s) in which this UMLObject is contained
  * as a text.
@@ -831,39 +838,41 @@ bool UMLObject::resolveRef()
     return true;
 }
 
-void UMLObject::saveToXMI1(QDomDocument &qDoc, QDomElement &qElement)
+void UMLObject::saveToXMI1(QXmlStreamWriter& writer)
 {
-    Q_UNUSED(qDoc); Q_UNUSED(qElement);
+    Q_UNUSED(writer);
 }
 
 /**
  * Auxiliary to saveToXMI1.
- * Create a QDomElement with the given tag, and save the XMI attributes
+ * Create an XML element with the given tag, and save the XMI attributes
  * that are common to all child classes to the newly created element.
  * This method does not need to be overridden by child classes.
+ * It is public because UMLOperation::saveToXMI1 invokes it for its
+ * <UML:Parameter>s (cannot be done with protected access).
  */
-QDomElement UMLObject::save1(const QString &tag, QDomDocument & qDoc)
+void UMLObject::save1(const QString& tag, QXmlStreamWriter& writer)
 {
     m_d->isSaved = true;
     /*
       Call as the first action of saveToXMI1() in child class:
-      This creates the QDomElement with which to work.
+      This creates the XML element with which to work.
     */
-    QDomElement qElement = qDoc.createElement(tag);
-    qElement.setAttribute(QLatin1String("isSpecification"), QLatin1String("false"));
+    writer.writeStartElement(tag);
+    writer.writeAttribute(QLatin1String("isSpecification"), QLatin1String("false"));
     if (m_BaseType != ot_Association &&
         m_BaseType != ot_Role &&
         m_BaseType != ot_Attribute &&
         m_BaseType != ot_Instance) {
-        qElement.setAttribute(QLatin1String("isLeaf"), QLatin1String("false"));
-        qElement.setAttribute(QLatin1String("isRoot"), QLatin1String("false"));
+        writer.writeAttribute(QLatin1String("isLeaf"), QLatin1String("false"));
+        writer.writeAttribute(QLatin1String("isRoot"), QLatin1String("false"));
         if (m_bAbstract)
-            qElement.setAttribute(QLatin1String("isAbstract"), QLatin1String("true"));
+            writer.writeAttribute(QLatin1String("isAbstract"), QLatin1String("true"));
         else
-            qElement.setAttribute(QLatin1String("isAbstract"), QLatin1String("false"));
+            writer.writeAttribute(QLatin1String("isAbstract"), QLatin1String("false"));
     }
-    qElement.setAttribute(QLatin1String("xmi.id"), Uml::ID::toString(m_nId));
-    qElement.setAttribute(QLatin1String("name"), m_name);
+    writer.writeAttribute(QLatin1String("xmi.id"), Uml::ID::toString(m_nId));
+    writer.writeAttribute(QLatin1String("name"), m_name);
     if (m_BaseType != ot_Operation &&
         m_BaseType != ot_Role &&
         m_BaseType != ot_Attribute) {
@@ -872,24 +881,57 @@ QDomElement UMLObject::save1(const QString &tag, QDomDocument & qDoc)
             nmSpc = umlPackage()->id();
         else
             nmSpc = UMLApp::app()->document()->modelID();
-        qElement.setAttribute(QLatin1String("namespace"), Uml::ID::toString(nmSpc));
+        writer.writeAttribute(QLatin1String("namespace"), Uml::ID::toString(nmSpc));
     }
     if (! m_Doc.isEmpty())
-        qElement.setAttribute(QLatin1String("comment"), m_Doc);    //CHECK: uml13.dtd compliance
+        writer.writeAttribute(QLatin1String("comment"), m_Doc);    //CHECK: uml13.dtd compliance
 #ifdef XMI_FLAT_PACKAGES
     if (umlParent()->asUMLPackage())             //FIXME: uml13.dtd compliance
-        qElement.setAttribute(QLatin1String("package"), umlParent()->asUMLPackage()->ID());
+        writer.writeAttribute(QLatin1String("package"), umlParent()->asUMLPackage()->ID());
 #endif
     QString visibility = Uml::Visibility::toString(m_visibility, false);
-    qElement.setAttribute(QLatin1String("visibility"), visibility);
+    writer.writeAttribute(QLatin1String("visibility"), visibility);
     if (m_pStereotype != 0)
-        qElement.setAttribute(QLatin1String("stereotype"), Uml::ID::toString(m_pStereotype->id()));
+        writer.writeAttribute(QLatin1String("stereotype"), Uml::ID::toString(m_pStereotype->id()));
     if (m_bStatic)
-        qElement.setAttribute(QLatin1String("ownerScope"), QLatin1String("classifier"));
+        writer.writeAttribute(QLatin1String("ownerScope"), QLatin1String("classifier"));
     /* else
-        qElement.setAttribute("ownerScope", "instance");
+        writer.writeAttribute("ownerScope", "instance");
      *** ownerScope defaults to instance if not set **********/
-    return qElement;
+}
+
+/**
+ * Auxiliary to saveToXMI1.
+ * Save possible stereotype tagged values stored in m_TaggedValues
+ * and write the XML end element created in save1().
+ */
+void UMLObject::save1end(QXmlStreamWriter& writer)
+{
+    // Save optional stereotype attributes
+    if (m_TaggedValues.count()) {
+        if (m_pStereotype == 0) {
+            uError() << m_name << " TaggedValues are set but pStereotype is null : clearing TaggedValues";
+            m_TaggedValues.clear();
+            return;
+        }
+        writer.writeStartElement(QLatin1String("UML:ModelElement.taggedValues"));
+        writer.writeAttribute(QLatin1String("stereotype"), Uml::ID::toString(m_pStereotype->id()));
+        const UMLStereotype::AttributeDefs& attrDefs = m_pStereotype->getAttributeDefs();
+        for (int i = 0; i < m_TaggedValues.count(); i++) {
+            if (i >= attrDefs.count()) {
+                uError() << m_name << ": stereotype " << m_pStereotype->name() << " defines "
+                         << attrDefs.count() << " attributes; ignoring excess TaggedValues";
+                break;
+            }
+            const QString& tv = m_TaggedValues.at(i);
+            writer.writeStartElement(QLatin1String("UML:TaggedValue"));
+            writer.writeAttribute(QLatin1String("tag"), attrDefs[i].name);
+            writer.writeAttribute(QLatin1String("value"), tv);
+            writer.writeEndElement();            // UML:TaggedValue
+        }
+        writer.writeEndElement();            // UML:ModelElement.taggedValues
+    }
+    writer.writeEndElement();
 }
 
 /**
@@ -1064,7 +1106,24 @@ bool UMLObject::loadFromXMI1(QDomElement & element)
         QDomElement elem = node.toElement();
         while (!elem.isNull()) {
             QString tag = elem.tagName();
-            if (UMLDoc::tagEq(tag, QLatin1String("name"))) {
+            if (UMLDoc::tagEq(tag, QLatin1String("ModelElement.taggedValues"))) {
+                QDomNode tvNode = elem.firstChild();
+                QDomElement tvElem = tvNode.toElement();
+                while (!tvElem.isNull()) {
+                    tag = tvElem.tagName();
+                    if (UMLDoc::tagEq(tag, QLatin1String("TaggedValue"))) {
+                        QString value = tvElem.attribute(QLatin1String("value"));
+                        m_TaggedValues.append(value);
+                        uDebug() << "loadFromXMI1(" << m_name
+                                 << "): Loaded " << tag << " value " << value;
+                    } else {
+                        uDebug() << "loadFromXMI1(" << m_name
+                                 << "): Unknown ModelElement.taggedValues child " << tag;
+                    }
+                    tvNode = tvNode.nextSibling();
+                    tvElem = tvNode.toElement();
+                }
+            } else if (UMLDoc::tagEq(tag, QLatin1String("name"))) {
                 m_name = elem.attribute(QLatin1String("xmi.value"));
                 if (m_name.isEmpty())
                     m_name = elem.text();
@@ -1188,7 +1247,7 @@ QString UMLObject::toI18nString(ObjectType t)
 
     default:
         name = QLatin1String("<unknown> &name:");
-        uWarning() << "unknown object type";
+        uWarning() << "UMLObject::toI18nString unknown object type " << toString(t);
         break;
     }
     return name;
@@ -1246,16 +1305,41 @@ Icon_Utils::IconType UMLObject::toIcon(ObjectType t)
     case  UMLObject::ot_Port:
         icon = Icon_Utils::it_Port;
         break;
-//    case  UMLObject::ot_Stereotype:
-//        icon = Icon_Utils::it_Stereotype;
-//        break;
+    case  UMLObject::ot_EnumLiteral:
+        icon = Icon_Utils::it_Enum_Literal;
+        break;
+    case  UMLObject::ot_Attribute:
+    case  UMLObject::ot_InstanceAttribute:
+        icon = Icon_Utils::it_Public_Attribute;
+        break;
+    case  UMLObject::ot_Operation:
+        icon = Icon_Utils::it_Public_Method;
+        break;
+    case  UMLObject::ot_Template:
+        icon = Icon_Utils::it_Template;
+        break;
+    case  UMLObject::ot_Category:
+        icon = Icon_Utils::it_Category;
+        break;
+    case  UMLObject::ot_EntityAttribute:
+        icon = Icon_Utils::it_Entity_Attribute;
+        break;
+    case  UMLObject::ot_UniqueConstraint:
+        icon = Icon_Utils::it_Unique_Constraint;
+        break;
+    case  UMLObject::ot_ForeignKeyConstraint:
+        icon = Icon_Utils::it_ForeignKey_Constraint;
+        break;
+    case  UMLObject::ot_CheckConstraint:
+        icon = Icon_Utils::it_Check_Constraint;
+        break;
     case  UMLObject::ot_UseCase:
         icon = Icon_Utils::it_UseCase;
         break;
 
     default:
         icon = Icon_Utils::it_Home;
-        uWarning() << "unknown object type";
+        uWarning() << "UMLObject::toIcon unknown object type " << toString(t);
         break;
     }
     return icon;
