@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2014                                               *
+ *   copyright (C) 2002-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -23,11 +23,11 @@ class AssociationWidget;
 class QDomDocument;
 class QDomElement;
 class QPainter;
+class QXmlStreamWriter;
 
 /**
- * This class provides with various symbols that can be embedded in
- * AssociationLine.  It also provides with convenience methods to align
- * the symbol to AssociationLine.
+ * This class provides various symbols that can be embedded in AssociationLine.
+ * It also provides convenience methods to align the symbol to AssociationLine.
  */
 class Symbol : public QGraphicsItem
 {
@@ -100,7 +100,7 @@ class Symbol : public QGraphicsItem
  *
  * @author Gopala Krishna
  * @author Andi Fischer
- * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
+ * Bugs and comments to uml-devel@lists.sf.net or https://bugs.kde.org
  */
 class AssociationLine : public QGraphicsObject
 {
@@ -117,6 +117,7 @@ public:
     QPointF startPoint() const;
     QPointF endPoint() const;
 
+    void addPoint(const QPointF& point);
     void insertPoint(int index, const QPointF& point);
     void removePoint(int index);
 
@@ -139,7 +140,7 @@ public:
     void dumpPoints();
 
     bool loadFromXMI1(QDomElement &qElement);
-    void saveToXMI1(QDomDocument &qDoc, QDomElement &qElement);
+    void saveToXMI1(QXmlStreamWriter& writer);
 
     QBrush brush() const;
     QPen pen() const;
@@ -182,7 +183,7 @@ private:
 
     void createSplinePoints();
 
-    AssociationWidget *m_associationWidget;      ///< association widget for which this line represents
+    AssociationWidget *m_associationWidget;      ///< association widget which this line represents
     QVector<QPointF>   m_points;                 ///< points representing the association line
     int                m_activePointIndex;       ///< index of active point which can be dragged to modify association line
     int                m_activeSegmentIndex;     ///< index of active segment
@@ -197,7 +198,7 @@ private:
     static QPainterPath createBezierCurve(QVector<QPointF> points);
     static QPainterPath createOrthogonalPath(QVector<QPointF> points);
 
-    qreal              c1dx, c1dy, c2dx, c2dy;
+    qreal              c1dx{0.0}, c1dy{0.0}, c2dx{0.0}, c2dy{0.0};
     static const qreal Delta;  ///< default delta for fuzzy recognition of points closer to point
     static const qreal SelectedPointDiameter;         ///< radius of circles drawn to show "selection"
     static const qreal SelfAssociationMinimumHeight;  ///< minimum height for self association's loop
