@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2014                                               *
+ *   copyright (C) 2002-2020                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -14,12 +14,15 @@
 //app includes
 #include "classifier.h"
 #include "debug_utils.h"
+#include "dialog_utils.h"
+#include "docwindow.h"
 #include "floatingtextwidget.h"
 #include "listpopupmenu.h"
 #include "objectwidget.h"
 #include "operation.h"
 #include "uml.h"
 #include "umldoc.h"
+#include "messagewidgetpropertiesdialog.h"
 #include "umlview.h"
 #include "uniqueid.h"
 #include "idchangelog.h"
@@ -29,6 +32,7 @@
 #include <QPainter>
 #include <QPolygon>
 #include <QResizeEvent>
+#include <QXmlStreamWriter>
 
 //kde includes
 #include <KLocalizedString>
@@ -62,11 +66,13 @@ MessageWidget::MessageWidget(UMLScene * scene, ObjectWidget* a, ObjectWidget* b,
     if (m_sequenceMessageType == Uml::SequenceMessage::Creation) {
         y -= m_pOw[Uml::RoleType::B]->height() / 2;
         m_pOw[Uml::RoleType::B]->setY(y);
-    }
+    } else if (m_sequenceMessageType == Uml::SequenceMessage::Destroy)
+        m_pOw[Uml::RoleType::B]->setShowDestruction(true);
     updateResizability();
     calculateWidget();
     y = y < getMinY() ? getMinY() : y;
-    y = y > getMaxY() ? getMaxY() : y;
+    if (y > b->getEndLineY())
+        b->setEndLine(y);
     setY(y);
 
     this->activate();
@@ -139,6 +145,8 @@ void MessageWidget::init()
  */
 MessageWidget::~MessageWidget()
 {
+    if (m_pOw[Uml::RoleType::B] && m_sequenceMessageType == Uml::SequenceMessage::Destroy)
+        m_pOw[Uml::RoleType::B]->setShowDestruction(false);
 }
 
 /**
@@ -180,14 +188,51 @@ void MessageWidget::updateResizability()
 
 /**
  * Overridden from UMLWidget.
- * Returns the cursor to be shown when resizing the widget.
- * The cursor shown is KCursor::sizeVerCursor().
+ * Checks if the mouse is in resize area and sets the cursor accordingly.
+ * The resize area is usually at the right bottom corner of the widget
+ * except in case of a message widget running from right to left.
+ * In that case the resize area is at the left bottom corner in order
+ * to avoid overlap with an execution rectangle at the right.
  *
- * @return The cursor to be shown when resizing the widget.
+ * @param me The QMouseEVent to check.
+ * @return true if the mouse is in resize area, false otherwise.
  */
-QCursor MessageWidget::resizeCursor() const
+bool MessageWidget::isInResizeArea(QGraphicsSceneMouseEvent *me)
 {
-    return Qt::SizeVerCursor;
+    if (!m_resizable) {
+        m_scene->activeView()->setCursor(Qt::ArrowCursor);
+        DEBUG(DBG_SRC) << "!m_resizable";
+        return false;
+    }
+
+    qreal m = 7.0;
+    const qreal w = width();
+    const qreal h = height();
+
+    // If the widget itself is very small then make the resize area small, too.
+    // Reason: Else it becomes impossible to do a move instead of resize.
+    if (w - m < m || h - m < m) {
+        m = 2.0;
+    }
+
+    if (me->scenePos().y() < y() + h - m) {
+        m_scene->activeView()->setCursor(Qt::ArrowCursor);
+        DEBUG(DBG_SRC) << "Y condition not satisfied";
+        return false;
+    }
+
+    int x1 = m_pOw[Uml::RoleType::A]->x();
+    int x2 = m_pOw[Uml::RoleType::B]->x();
+    if ((x1 < x2 && me->scenePos().x() >= x() + w - m) ||
+        (x1 > x2 && me->scenePos().x() >= x() - m)) {
+        m_scene->activeView()->setCursor(Qt::SizeVerCursor);
+        DEBUG(DBG_SRC) << "X condition is satisfied";
+        return true;
+    } else {
+        m_scene->activeView()->setCursor(Qt::ArrowCursor);
+        DEBUG(DBG_SRC) << "X condition not satisfied";
+        return false;
+    }
 }
 
 /**
@@ -224,9 +269,9 @@ void MessageWidget::resizeWidget(qreal newW, qreal newH)
 
 /**
  * Constrains the vertical position of the message widget so it doesn't go
- * upper than the bottom side of the lower object.
- * The height of the floating text widget in the message is taken in account
- * if there is any and isn't empty.
+ * above the bottom side of the lower object.
+ * The height of the floating text widget in the message is taken into account
+ * if there is any and it isn't empty.
  *
  * @param diffY The difference between current Y position and new Y position.
  * @return The new Y position, constrained.
@@ -303,6 +348,8 @@ void MessageWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *opt
         paintAsynchronous(painter, option);
     } else if (m_sequenceMessageType == Uml::SequenceMessage::Creation) {
         paintCreation(painter, option);
+    } else if (m_sequenceMessageType == Uml::SequenceMessage::Destroy) {
+        paintDestroy(painter, option);
     } else if (m_sequenceMessageType == Uml::SequenceMessage::Lost) {
         paintLost(painter, option);
     } else if (m_sequenceMessageType == Uml::SequenceMessage::Found) {
@@ -498,7 +545,7 @@ void MessageWidget::paintCreation(QPainter *painter, const QStyleOptionGraphicsI
             offsetX += 7;
             w -= 7;
         }
-        paintArrow(painter, offsetX, lineY, w, Qt::RightArrow);
+        paintArrow(painter, offsetX, lineY, w, Qt::RightArrow, true);
         if (messageOverlapsA) {
             offsetX -= 7;
         }
@@ -506,12 +553,16 @@ void MessageWidget::paintCreation(QPainter *painter, const QStyleOptionGraphicsI
         if (messageOverlapsA) {
             w -= 7;
         }
-        paintArrow(painter, offsetX, lineY, w, Qt::LeftArrow);
+        paintArrow(painter, offsetX, lineY, w, Qt::LeftArrow, true);
     }
 
     UMLWidget::paint(painter, option);
 }
 
+void MessageWidget::paintDestroy(QPainter *painter, const QStyleOptionGraphicsItem *option)
+{
+    paintSynchronous(painter, option);
+}
 
 /**
  * Draws a solid arrow line and a stick arrow head
@@ -772,8 +823,10 @@ void MessageWidget::slotMenuSelection(QAction* action)
 {
     ListPopupMenu::MenuType sel = ListPopupMenu::typeFromAction(action);
     if (sel == ListPopupMenu::mt_Delete) {
-        // This will clean up this widget and the text widget:
-        m_scene->removeWidget(this);
+        if (Dialog_Utils::askDeleteAssociation()) {
+            // This will clean up this widget and the text widget:
+            m_scene->removeWidget(this);
+        }
     } else {
 
         UMLWidget::slotMenuSelection(action);
@@ -1016,6 +1069,8 @@ void MessageWidget::calculateDimensions()
         calculateDimensionsAsynchronous();
     } else if (m_sequenceMessageType == Uml::SequenceMessage::Creation) {
         calculateDimensionsCreation();
+    } else if (m_sequenceMessageType == Uml::SequenceMessage::Destroy) {
+        calculateDimensionsDestroy();
     } else if (m_sequenceMessageType == Uml::SequenceMessage::Lost) {
         calculateDimensionsLost();
     } else if (m_sequenceMessageType == Uml::SequenceMessage::Found) {
@@ -1129,6 +1184,14 @@ void MessageWidget::calculateDimensionsCreation()
     setSize(widgetWidth, widgetHeight);
 }
 
+/**
+ * Calculates and sets the size of the widget for a destroy message.
+ */
+void MessageWidget::calculateDimensionsDestroy()
+{
+    calculateDimensionsSynchronous();
+}
+
 /**
  * Calculates and sets the size of the widget for a lost message.
  */
@@ -1269,6 +1332,8 @@ QSizeF MessageWidget::minimumSize() const
         return isSelf() ? QSizeF(width(), 20) : QSizeF(width(), 8);
     } else if (m_sequenceMessageType == Uml::SequenceMessage::Creation) {
         return QSizeF(width(), 8);
+    } else if (m_sequenceMessageType == Uml::SequenceMessage::Destroy) {
+        return QSizeF(width(), 8);
     } else if (m_sequenceMessageType == Uml::SequenceMessage::Lost) {
         return QSizeF(width(), 10);
     } else if (m_sequenceMessageType == Uml::SequenceMessage::Found) {
@@ -1318,50 +1383,57 @@ void MessageWidget::setyclicked(int yclick)
 }
 
 /**
- * Event handler for mouse double click events.
- * @param event QGraphicsSceneMouseEvent which triggered the double click event
+ * Show a properties dialog for an ObjectWidget.
  */
-void MessageWidget::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
+bool MessageWidget::showPropertiesDialog()
 {
-    Q_UNUSED(event)
-    DEBUG(DBG_SRC) << "NOT IMPLEMENTED YET!";
-    if (m_pFText != 0) {
-        DEBUG(DBG_SRC) << "NOT IMPLEMENTED YET - on floating text widget!";
-//        QAction* action = m_pMenu->getAction(ListPopupMenu::mt_Select_Operation);
-//        m_pFText->slotMenuSelection(action);
+    if (!lwClassifier()) {
+        uError() << "lwClassifier() returns a NULL classifier";
+        return false;
     }
+    bool result = false;
+    UMLApp::app()->docWindow()->updateDocumentation(false);
+    QPointer<MessageWidgetPropertiesDialog> dlg = new MessageWidgetPropertiesDialog(0, this);
+    if (dlg->exec()) {
+        m_pFText->setMessageText();
+        UMLApp::app()->docWindow()->showDocumentation(this, true);
+        UMLApp::app()->document()->setModified(true);
+        result = true;
+    }
+    delete dlg;
+    return result;
 }
 
 /**
  * Saves to the "messagewidget" XMI element.
  */
-void MessageWidget::saveToXMI1(QDomDocument & qDoc, QDomElement & qElement)
+void MessageWidget::saveToXMI1(QXmlStreamWriter& writer)
 {
-    QDomElement messageElement = qDoc.createElement(QLatin1String("messagewidget"));
-    UMLWidget::saveToXMI1(qDoc, messageElement);
-    LinkWidget::saveToXMI1(qDoc, messageElement);
+    writer.writeStartElement(QLatin1String("messagewidget"));
+    UMLWidget::saveToXMI1(writer);
+    LinkWidget::saveToXMI1(writer);
     if (m_pOw[Uml::RoleType::A])
-        messageElement.setAttribute(QLatin1String("widgetaid"), Uml::ID::toString(m_pOw[Uml::RoleType::A]->localID()));
+        writer.writeAttribute(QLatin1String("widgetaid"), Uml::ID::toString(m_pOw[Uml::RoleType::A]->localID()));
     if (m_pOw[Uml::RoleType::B])
-        messageElement.setAttribute(QLatin1String("widgetbid"), Uml::ID::toString(m_pOw[Uml::RoleType::B]->localID()));
+        writer.writeAttribute(QLatin1String("widgetbid"), Uml::ID::toString(m_pOw[Uml::RoleType::B]->localID()));
     UMLOperation *pOperation = operation();
     if (pOperation)
-        messageElement.setAttribute(QLatin1String("operation"), Uml::ID::toString(pOperation->id()));
+        writer.writeAttribute(QLatin1String("operation"), Uml::ID::toString(pOperation->id()));
     else
-        messageElement.setAttribute(QLatin1String("operation"), m_CustomOp);
-    messageElement.setAttribute(QLatin1String("sequencemessagetype"), m_sequenceMessageType);
+        writer.writeAttribute(QLatin1String("operation"), m_CustomOp);
+    writer.writeAttribute(QLatin1String("sequencemessagetype"), QString::number(m_sequenceMessageType));
     if (m_sequenceMessageType == Uml::SequenceMessage::Lost || m_sequenceMessageType == Uml::SequenceMessage::Found) {
-        messageElement.setAttribute(QLatin1String("xclicked"), m_xclicked);
-        messageElement.setAttribute(QLatin1String("yclicked"), m_yclicked);
+        writer.writeAttribute(QLatin1String("xclicked"), QString::number(m_xclicked));
+        writer.writeAttribute(QLatin1String("yclicked"), QString::number(m_yclicked));
     }
 
     // save the corresponding message text
     if (m_pFText && !m_pFText->text().isEmpty()) {
-        messageElement.setAttribute(QLatin1String("textid"), Uml::ID::toString(m_pFText->id()));
-        m_pFText->saveToXMI1(qDoc, messageElement);
+        writer.writeAttribute(QLatin1String("textid"), Uml::ID::toString(m_pFText->id()));
+        m_pFText->saveToXMI1(writer);
     }
 
-    qElement.appendChild(messageElement);
+    writer.writeEndElement();
 }
 
 /**
