@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2014-2020                                               *
+ *   copyright (C) 2014-2021                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -39,6 +39,10 @@ public:
 
     void setInitialPosition(const QPointF &scenePos);
 
+    qreal getX() const;
+    qreal getY() const;
+    QPointF getPos() const;
+
     void updateWidget();
     void setName(const QString &strName);
     void moveWidgetBy(qreal diffX, qreal diffY);
@@ -50,7 +54,7 @@ public:
     FloatingTextWidget *floatingTextWidget();
     void setFloatingTextWidget(FloatingTextWidget *ft);
 
-    void saveToXMI1(QDomDocument& qDoc, QDomElement& qElement);
+    void saveToXMI1(QXmlStreamWriter& writer);
     bool loadFromXMI1(QDomElement& qElement);
 
 public slots:
