@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2020                                               *
+ *   copyright (C) 2002-2021                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -42,6 +42,7 @@
 #include "signalwidget.h"
 #include "settingsdialog.h"
 #include "statewidget.h"
+#include "stereotype.h"
 #include "uml.h"
 #include "umldoc.h"
 #include "umllistview.h"
@@ -63,6 +64,7 @@
 #include <QColor>
 #include <QPainter>
 #include <QPointer>
+#include <QXmlStreamWriter>
 
 using namespace Uml;
 
@@ -367,6 +369,15 @@ void UMLWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
         event->ignore();
         return;
     }
+    DEBUG(DBG_SRC) << "widget = " << name() << " / type = " << baseTypeStr()
+                   << " event->scenePos = " << event->scenePos()
+                   << " pos = " << pos();
+    /*
+    if (! onWidget(event->scenePos())) {
+        DEBUG(DBG_SRC) << name() << " event->scenePos onWidget = false, ignoring event";
+        event->ignore();
+        return;
+    } */
     event->accept();
     DEBUG(DBG_SRC) << "widget = " << name() << " / type = " << baseTypeStr();
 
@@ -396,7 +407,7 @@ void UMLWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
 
     m_shiftPressed = false;
 
-    int count = m_scene->selectedCount(true);
+    int count = m_scene->selectedCount();
     if (event->button() == Qt::LeftButton) {
         if (isSelected() && count > 1) {
             // single selection is made in release event if the widget wasn't moved
@@ -429,14 +440,14 @@ void UMLWidget::mousePressEvent(QGraphicsSceneMouseEvent *event)
  * widget resize constraint can be applied), and then the associations are
  * adjusted.
  * The resizing can be constrained also to a specific axis using control
- * and shift buttons. If on or another is pressed, it's constrained to X axis.
+ * and shift buttons. If one or another is pressed, it's constrained to X axis.
  * If both are pressed, it's constrained to Y axis.
  *
  * If not resizing, the widget is being moved. If the move is being started,
  * the selection bounds are set (which includes updating the list of selected
  * widgets).
  * The difference between the previous position of the selection and the new
- * one is got (taking in account the selection bounds so widgets don't go
+ * one is calculated (taking in account the selection bounds so widgets don't go
  * beyond the scene limits). Then, it's constrained to X or Y axis depending
  * on shift and control buttons.
  * A further constraint is made using constrainMovementForAllWidgets (for example,
@@ -459,7 +470,8 @@ void UMLWidget::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
     }
 
     if (!m_moved) {
-        UMLApp::app()->document()->writeToStatusBar(i18n("Hold shift or ctrl to move in X axis. Hold shift and control to move in Y axis. Right button click to cancel move."));
+        UMLApp::app()->document()->writeToStatusBar
+          (i18n("Hold shift or ctrl to move in X axis. Hold shift and control to move in Y axis. Right button click to cancel move."));
 
         m_moved = true;
         //Maybe needed by AssociationWidget
@@ -534,7 +546,7 @@ void UMLWidget::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
 void UMLWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
 {
     if (!m_moved && !m_resized) {
-        if (!m_shiftPressed && (m_scene->selectedCount(true) > 1)) {
+        if (!m_shiftPressed && (m_scene->selectedCount() > 1)) {
             selectSingle(event);
         } else if (!isSelected()) {
             deselect(event);
@@ -557,6 +569,7 @@ void UMLWidget::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
             UMLApp::app()->executeCommand(new Uml::CmdResizeWidget(this));
             m_autoResize = false;
             m_resized = false;
+            deselect(event);
         }
 
         if ((m_inMoveArea && wasPositionChanged()) ||
@@ -720,7 +733,7 @@ void UMLWidget::init()
         m_useFillColor = false;
         m_usesDiagramFillColor = false;
         m_usesDiagramUseFillColor = false;
-        m_showStereotype = false;
+        m_showStereotype = Uml::ShowStereoType::None;
     }
 
     m_resizable = true;
@@ -907,7 +920,7 @@ void UMLWidget::slotMenuSelection(QAction *trigger)
     case ListPopupMenu::mt_MessageDestroy:
     case ListPopupMenu::mt_MessageSynchronous:
 //        MessageWidget *widget = new MessageWidget(umlScene(), this);
-//        addConnectedWidget(widget, Uml::AssociationType::Coll_Message_Synchronous);
+//        addConnectedWidget(widget, Uml::AssociationType::Coll_Mesg_Sync);
     case ListPopupMenu::mt_MessageAsynchronous:
     case ListPopupMenu::mt_MessageFound:
     case ListPopupMenu::mt_MessageLost:
@@ -1675,6 +1688,39 @@ void UMLWidget::setScene(UMLScene *scene)
     connect(m_scene, SIGNAL(sigLineWidthChanged(Uml::ID::Type)), this, SLOT(slotLineWidthChanged(Uml::ID::Type)));
 }
 
+/**
+ * Gets the x-coordinate.
+ * Currently, the only class that reimplements this method is PinPortBase.
+ *
+ * @return The x-coordinate.
+ */
+qreal UMLWidget::getX() const
+{
+    return QGraphicsObjectWrapper::x();
+}
+
+/**
+ * Gets the y-coordinate.
+ * Currently, the only class that reimplements this method is PinPortBase.
+ *
+ * @return The y-coordinate.
+ */
+qreal UMLWidget::getY() const
+{
+    return QGraphicsObjectWrapper::y();
+}
+
+/**
+ * Gets the position.
+ * Currently, the only class that reimplements this method is PinPortBase.
+ *
+ * @return The QGraphicsObject position.
+ */
+QPointF UMLWidget::getPos() const
+{
+    return QGraphicsObjectWrapper::pos();
+}
+
 /**
  * Sets the x-coordinate.
  * Currently, the only class that reimplements this method is
@@ -1885,11 +1931,24 @@ void UMLWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
         // resize anchor would cover up most of the widget.
         if (m_resizable && w >= s+8 && h >= s+8) {
             brush.setColor(Qt::red);
-            const int right = 0 + w;
             const int bottom = 0 + h;
-            painter->drawLine(right - s, 0 + h - 1, 0 + w - 1, 0 + h - s);
-            painter->drawLine(right - (s*2), bottom - 1, right - 1, bottom - (s*2));
-            painter->drawLine(right - (s*3), bottom - 1, right - 1, bottom - (s*3));
+            int horSide = w;   // horizontal side default: right side
+            if (baseType() == wt_Message) {
+               MessageWidget *msg = asMessageWidget();
+               int x1 = msg->objectWidget(Uml::RoleType::A)->x();
+               int x2 = msg->objectWidget(Uml::RoleType::B)->x();
+               if (x1 > x2) {
+                   // On messages running right to left we use the left side for
+                   // placing the resize anchor because the message's execution
+                   // specification as at the left in this case.  Furthermore,
+                   // the right side may be covered up by another message's
+                   // execution specification.
+                   horSide = 17;  // execution box width
+               }
+            }
+            painter->drawLine(horSide - s, 0 + h - 1, 0 + w - 1, 0 + h - s);
+            painter->drawLine(horSide - (s*2), bottom - 1, horSide - 1, bottom - (s*2));
+            painter->drawLine(horSide - (s*3), bottom - 1, horSide - 1, bottom - (s*3));
         } else {
             painter->fillRect(0 + w - s, 0 + h - s, s, s, brush);
         }
@@ -2013,21 +2072,62 @@ void UMLWidget::forceUpdateFontMetrics(QFont& font, QPainter *painter)
 /**
  * Set the status of whether to show Stereotype.
  *
- * @param flag   True if stereotype shall be shown.
+ * @param flag   Value of type Uml::ShowStereoType::Enum
  */
-void UMLWidget::setShowStereotype(bool flag)
+void UMLWidget::setShowStereotype(Uml::ShowStereoType::Enum flag)
 {
     m_showStereotype = flag;
     updateGeometry();
     update();
 }
 
+/**
+ * Return stereotype concrete attributes concatenated into single string
+ * with the attribute name given before each value and delimited by "{"
+ * at start and "}" at end.
+ * Example:
+ * For a stereotype with attribute 'foo' of type Double and attribute 'bar'
+ * of type String and concrete values 1.0 for 'foo' and "hello" for 'bar',
+ * the result is:  {foo=1.0,bar="hello"}
+ */
+QString UMLWidget::tags() const
+{
+    if (m_umlObject == 0)
+        return QString();
+    UMLStereotype *s = m_umlObject->umlStereotype();
+    if (s == 0)
+        return QString();
+    UMLStereotype::AttributeDefs adefs = s->getAttributeDefs();
+    if (adefs.isEmpty())
+        return QString();
+    const QStringList& umlTags = m_umlObject->tags();
+    QString taggedValues(QLatin1String("{"));
+    for (int i = 0; i < adefs.size(); i++) {
+        UMLStereotype::AttributeDef ad = adefs.at(i);
+        taggedValues.append(ad.name);
+        taggedValues.append(QLatin1String("="));
+        QString value = ad.defaultVal;
+        if (i < umlTags.size()) {
+            QString umlTag = umlTags.at(i);
+            if (!umlTag.isEmpty())
+                value = umlTag;
+        }
+        if (ad.type == Uml::PrimitiveTypes::String)
+            value = QLatin1String("\"") + value + QLatin1String("\"");
+        taggedValues.append(value);
+        if (i < adefs.size() - 1)
+            taggedValues.append(QLatin1String(","));
+     }
+     taggedValues.append(QLatin1String("}"));
+     return taggedValues;
+}
+
 /**
  * Returns the status of whether to show Stereotype.
  *
  * @return  True if stereotype is shown.
  */
-bool UMLWidget::showStereotype() const
+Uml::ShowStereoType::Enum UMLWidget::showStereotype() const
 {
     return m_showStereotype;
 }
@@ -2042,26 +2142,27 @@ void UMLWidget::moveEvent(QGraphicsSceneMouseEvent* me)
   Q_UNUSED(me)
 }
 
-void UMLWidget::saveToXMI1(QDomDocument & qDoc, QDomElement & qElement)
+void UMLWidget::saveToXMI1(QXmlStreamWriter& writer)
 {
     /*
-      Call after required actions in child class.
+      When calling this from child classes bear in mind that the call
+      must precede terminated XML subelements.
       Type must be set in the child class.
     */
-    WidgetBase::saveToXMI1(qDoc, qElement);
-    DiagramProxyWidget::saveToXMI1(qDoc, qElement);
+    WidgetBase::saveToXMI1(writer);
+    DiagramProxyWidget::saveToXMI1(writer);
 
     qreal dpiScale = UMLApp::app()->document()->dpiScale();
-    qElement.setAttribute(QLatin1String("x"), QString::number(x() / dpiScale));
-    qElement.setAttribute(QLatin1String("y"), QString::number(y() / dpiScale));
-    qElement.setAttribute(QLatin1String("width"), QString::number(width() / dpiScale));
-    qElement.setAttribute(QLatin1String("height"), QString::number(height() / dpiScale));
+    writer.writeAttribute(QLatin1String("x"), QString::number(x() / dpiScale));
+    writer.writeAttribute(QLatin1String("y"), QString::number(y() / dpiScale));
+    writer.writeAttribute(QLatin1String("width"), QString::number(width() / dpiScale));
+    writer.writeAttribute(QLatin1String("height"), QString::number(height() / dpiScale));
 
-    qElement.setAttribute(QLatin1String("isinstance"), m_isInstance);
+    writer.writeAttribute(QLatin1String("isinstance"), QString::number(m_isInstance));
     if (!m_instanceName.isEmpty())
-        qElement.setAttribute(QLatin1String("instancename"), m_instanceName);
+        writer.writeAttribute(QLatin1String("instancename"), m_instanceName);
     if (m_showStereotype)
-        qElement.setAttribute(QLatin1String("showstereotype"), m_showStereotype);
+        writer.writeAttribute(QLatin1String("showstereotype"), QString::number(m_showStereotype));
 }
 
 bool UMLWidget::loadFromXMI1(QDomElement & qElement)
@@ -2082,7 +2183,7 @@ bool UMLWidget::loadFromXMI1(QDomElement & qElement)
     m_isInstance = (bool)isinstance.toInt();
     m_instanceName = qElement.attribute(QLatin1String("instancename"));
     QString showstereo = qElement.attribute(QLatin1String("showstereotype"), QLatin1String("0"));
-    m_showStereotype = (bool)showstereo.toInt();
+    m_showStereotype = (Uml::ShowStereoType::Enum)showstereo.toInt();
 
     return true;
 }
