@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2020                                               *
+ *   copyright (C) 2002-2021                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -105,6 +105,9 @@ public:
         return m_startMove;
     }
 
+    virtual qreal getX() const;
+    virtual qreal getY() const;
+    virtual QPointF getPos() const;
     virtual void setX(qreal x);
     virtual void setY(qreal y);
 
@@ -166,8 +169,9 @@ public:
         return m_instanceName;
     }
 
-    bool showStereotype() const;
-    virtual void setShowStereotype(bool flag);
+    Uml::ShowStereoType::Enum showStereotype() const;
+    virtual void setShowStereotype(Uml::ShowStereoType::Enum flag);
+    QString tags() const;
 
     virtual bool showPropertiesDialog();
 
@@ -187,7 +191,7 @@ public:
     void forceUpdateFontMetrics(QFont &font, QPainter *painter);
 
     virtual bool loadFromXMI1(QDomElement &qElement);
-    virtual void saveToXMI1(QDomDocument &qDoc, QDomElement &qElement);
+    virtual void saveToXMI1(QXmlStreamWriter& writer);
 
     QPointF startMovePosition() const;
     void setStartMovePosition(const QPointF &position);
@@ -294,7 +298,7 @@ protected:
 
     QString m_instanceName;  ///< instance name (used if on a deployment diagram)
     bool m_isInstance;       ///< holds whether this widget is a component instance (i.e. on a deployment diagram)
-    bool m_showStereotype;   ///< should the stereotype be displayed
+    Uml::ShowStereoType::Enum m_showStereotype;   ///< if and how the stereotype should be displayed
 
     ///////////////// End of Data Loaded/Saved //////////////////////////
 
@@ -302,7 +306,7 @@ protected:
     QPointF        m_startMovePostion;
     QSizeF         m_startResizeSize;
     int            m_nPosX;
-    UMLDoc        *m_doc;  ///< shortcut for UMLApp::app()->getDocument()
+    UMLDoc        *m_doc;  ///< shortcut for UMLApp::app()->document()
     bool           m_resizable;
     QFontMetrics  *m_pFontMetrics[FT_INVALID];
     QSizeF         m_minimumSize;
