@@ -4,7 +4,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2014                                               *
+ *   copyright (C) 2002-2021                                               *
  *   Umbrello UML Modeller Authors <umbrello-devel@kde.org>                *
  ***************************************************************************/
 
@@ -91,7 +91,7 @@ void WorkToolBar::insertBasicAssociations()
         m_Type == Uml::DiagramType::Component || m_Type == Uml::DiagramType::Deployment) {
         insertHotBtn(tbb_UniAssociation);
     }
-    if(m_Type != Uml::DiagramType::Object){
+    if (m_Type != Uml::DiagramType::Object) {
         insertHotBtn(tbb_Dependency);
         insertHotBtn(tbb_Generalization);
     }
@@ -144,6 +144,8 @@ void WorkToolBar::slotCheckToolBar(Uml::DiagramType::Enum dt)
 
     case Uml::DiagramType::Sequence:
         insertHotBtn(tbb_Object);
+        insertHotBtn(tbb_Seq_Message_Creation);
+        insertHotBtn(tbb_Seq_Message_Destroy);
         insertHotBtn(tbb_Seq_Message_Synchronous);
         insertHotBtn(tbb_Seq_Message_Asynchronous);
         insertHotBtn(tbb_Seq_Message_Found);
@@ -154,8 +156,9 @@ void WorkToolBar::slotCheckToolBar(Uml::DiagramType::Enum dt)
 
     case Uml::DiagramType::Collaboration:
         insertHotBtn(tbb_Object);
-        insertHotBtn(tbb_Coll_Message_Asynchronous);
-        insertHotBtn(tbb_Coll_Message_Synchronous);
+        insertHotBtn(tbb_Instance);
+        insertHotBtn(tbb_Coll_Mesg_Async);
+        insertHotBtn(tbb_Coll_Mesg_Sync);
         break;
 
     case Uml::DiagramType::State:
@@ -327,6 +330,8 @@ void WorkToolBar::loadPixmaps()
     } buttonInfo[] = {
         { tbb_Arrow,                    i18nc("selection arrow", "Select"), Icon_Utils::it_Arrow,                SLOT(slotArrow()) },
         { tbb_Object,                   i18n("Object"),                  Icon_Utils::it_Object,                  SLOT(slotObject()) },
+        { tbb_Seq_Message_Creation,     i18n("Creation"),                Icon_Utils::it_Message_Creation,        SLOT(slotSeq_Message_Creation()) },
+        { tbb_Seq_Message_Destroy,      i18n("Destroy"),                 Icon_Utils::it_Message_Destroy,         SLOT(slotSeq_Message_Destroy()) },
         { tbb_Seq_Message_Synchronous,  i18n("Synchronous Message"),     Icon_Utils::it_Message_Sync,            SLOT(slotSeq_Message_Synchronous()) },
         { tbb_Seq_Message_Asynchronous, i18n("Asynchronous Message"),    Icon_Utils::it_Message_Async,           SLOT(slotSeq_Message_Asynchronous()) },
         { tbb_Seq_Message_Found,        i18n("Found Message"),           Icon_Utils::it_Message_Found,           SLOT(slotSeq_Message_Found()) },
@@ -385,8 +390,8 @@ void WorkToolBar::loadPixmaps()
         { tbb_Pin,                      i18n("Pin"),                     Icon_Utils::it_Pin,                     SLOT(slotPin()) },
         { tbb_Port,                     i18n("Port"),                    Icon_Utils::it_Pin,                     SLOT(slotPort()) },
         { tbb_Initial_Activity,         i18n("Initial Activity"),        Icon_Utils::it_Activity_Initial,        SLOT(slotInitial_Activity()) },
-        { tbb_Coll_Message_Synchronous, i18n("Synchronous Message"),     Icon_Utils::it_Message_Synchronous,     SLOT(slotColl_Message_Synchronous()) },
-        { tbb_Coll_Message_Asynchronous,i18n("Asynchronous Message"),    Icon_Utils::it_Message_Asynchronous,    SLOT(slotColl_Message_Asynchronous()) },
+        { tbb_Coll_Mesg_Sync,           i18n("Synchronous Message"),     Icon_Utils::it_Message_Synchronous,     SLOT(slotColl_Mesg_Sync()) },
+        { tbb_Coll_Mesg_Async,          i18n("Asynchronous Message"),    Icon_Utils::it_Message_Asynchronous,    SLOT(slotColl_Mesg_Async()) },
         { tbb_Exception,                i18n("Exception"),               Icon_Utils::it_Exception,               SLOT(slotException()) },
         { tbb_Object_Node,              i18n("Object Node"),             Icon_Utils::it_Object_Node,             SLOT(slotObject_Node()) },
         { tbb_PrePostCondition,         i18n("Pre/Post condition"),      Icon_Utils::it_Condition_PrePost,       SLOT(slotPrePostCondition()) },
@@ -428,8 +433,10 @@ void WorkToolBar::slotAggregation()              { buttonChanged(tbb_Aggregation
 void WorkToolBar::slotDependency()               { buttonChanged(tbb_Dependency);               }
 void WorkToolBar::slotAssociation()              { buttonChanged(tbb_Association);              }
 void WorkToolBar::slotContainment()              { buttonChanged(tbb_Containment);              }
-void WorkToolBar::slotColl_Message_Synchronous() { buttonChanged(tbb_Coll_Message_Synchronous); }
-void WorkToolBar::slotColl_Message_Asynchronous(){ buttonChanged(tbb_Coll_Message_Asynchronous);}
+void WorkToolBar::slotColl_Mesg_Sync()           { buttonChanged(tbb_Coll_Mesg_Sync);           }
+void WorkToolBar::slotColl_Mesg_Async()          { buttonChanged(tbb_Coll_Mesg_Async);          }
+void WorkToolBar::slotSeq_Message_Creation()     { buttonChanged(tbb_Seq_Message_Creation);     }
+void WorkToolBar::slotSeq_Message_Destroy()      { buttonChanged(tbb_Seq_Message_Destroy);      }
 void WorkToolBar::slotSeq_Message_Synchronous()  { buttonChanged(tbb_Seq_Message_Synchronous);  }
 void WorkToolBar::slotSeq_Message_Asynchronous() { buttonChanged(tbb_Seq_Message_Asynchronous); }
 void WorkToolBar::slotSeq_Message_Found()        { buttonChanged(tbb_Seq_Message_Found);        }
